﻿Type=Class
Version=0.1
ModulesStructureVersion=1
B4i=true
@EndOfDesignText@
'Class module
'v1.00
Sub Class_Globals
	Private fe = 500000, ok = 0.9996, deg2rad = cPI / 180, rad2deg = 1 / deg2rad As Double
	Type LatLon (Lat As Double, Lon As Double)
	Type UTM (UtmXZone As Int, X As Double, NorthHemisphere As Boolean, Y As Double)

End Sub

Public Sub Initialize

End Sub

Private Sub CalculateESquared(a As Double, b As Double) As Double
    Return ((a * a) - (b * b)) / (a * a)
End Sub

Private Sub CalculateE2Squared(a As Double, b As Double) As Double
	Return ((a * a) - (b * b)) / (b * b)
End Sub

Private Sub denom(es As Double,  sphi As Double) As Double
	Dim sinSphi As Double
	sinSphi = Sin(sphi)
	Return Sqrt(1.0 - es * (sinSphi * sinSphi))
End Sub

Private Sub sphsr( a As Double,  es As Double,  sphi As Double) As Double
	Dim dn As Double
	dn = denom(es, sphi)
	Return a * (1.0 - es) / (dn * dn * dn)
End Sub


Private Sub sphsn( a As Double,  es As Double,  sphi As Double) As Double
    Dim sinSphi As Double
    sinSphi = Sin(sphi)
    Return a / Sqrt(1.0 - es * (sinSphi * sinSphi))
End Sub

Private Sub sphtmd( ap As Double,  bp As Double,  cp As Double,  dp As Double,  ep As Double, _
 sphi As Double) As Double
   Return  (ap * sphi) - (bp * Sin(2.0 * sphi)) + (cp * Sin(4.0 * sphi)) _
        - (dp * Sin(6.0 * sphi)) + (ep * Sin(8.0 * sphi))

End Sub
'Convert LatLon to UTM using WGS84 datum
Public Sub WGS84LatLonToUTM(LL As LatLon) As UTM
    Return LatLonToUTM(6378137, 0.003352810665, LL)
End Sub
'Convert UTM to LatLon using WGS84 datum
Public Sub WGS84UTMTOLatLon(U As UTM) As LatLon
    Return UTMToLatLon(6378137, 0.003352810665, U)
End Sub

Private Sub Fix(d As Double) As Int
	Return d
End Sub

'Convert LatLon to UTM.
'a - Equatorial radius
'f - Flattening
Public Sub LatLonToUTM(a As Double,  f As Double,  LL As LatLon) As UTM
    Dim recf As Double
    Dim b As Double
    Dim eSquared As Double
    Dim e2Squared As Double
    Dim tn As Double
    Dim ap As Double
    Dim bp As Double
    Dim cp As Double
    Dim dp As Double
    Dim ep As Double
    Dim olam As Double
    Dim dlam As Double
    Dim s As Double
    Dim c As Double
    Dim t As Double
    Dim eta As Double
    Dim sn As Double
    Dim tmd As Double
    Dim t1 As Double
    Dim t2 As Double
    Dim t3 As Double
    Dim t6 As Double
    Dim t7 As Double
    Dim nfn As Double
	Dim Lat = LL.Lat, Lon = LL.Lon As Double
    Dim easting, northing, utmXZone As Double
    If (Lon <= 0) Then
        utmXZone = 30 + Fix(Lon / 6)
    Else
        utmXZone = 31 + Fix(Lon / 6)
    End If
	Dim res As UTM
	res.Initialize
    res.NorthHemisphere = Lat >= 0
    Dim latRad As Double
    latRad = Lat * deg2rad
    Dim lonRad As Double
    lonRad = Lon * deg2rad
    recf = 1 / f
    b = a * (recf - 1.0) / recf
    eSquared = CalculateESquared(a, b)
    e2Squared = CalculateE2Squared(a, b)
	
    tn = (a - b) / (a + b)
    ap = a * (1.0 - tn + 5.0 * ((tn * tn) - (tn * tn * tn)) / 4.0 + 81.0 * _
        ((tn * tn * tn * tn) - (tn * tn * tn * tn * tn)) / 64.0)
    bp = 3.0 * a * (tn - (tn * tn) + 7.0 * ((tn * tn * tn) _
        - (tn * tn * tn * tn)) / 8.0 + 55.0 * (tn * tn * tn * tn * tn) / 64.0) _
        / 2.0
    cp = 15.0 * a * ((tn * tn) - (tn * tn * tn) + 3.0 * ((tn * tn * tn * tn) _
        - (tn * tn * tn * tn * tn)) / 4.0) / 16.0
    dp = 35.0 * a * ((tn * tn * tn) - (tn * tn * tn * tn) + 11.0 _
        * (tn * tn * tn * tn * tn) / 16.0) / 48.0
    ep = 315.0 * a * ((tn * tn * tn * tn) - (tn * tn * tn * tn * tn)) / 512.0
    olam = (utmXZone * 6 - 183) * deg2rad
    dlam = lonRad - olam
    s = Sin(latRad)
    c = Cos(latRad)
    t = s / c
    eta = e2Squared * (c * c)
    sn = sphsn(a, eSquared, latRad)
    tmd = sphtmd(ap, bp, cp, dp, ep, latRad)
    t1 = tmd * ok
    t2 = sn * s * c * ok / 2.0
    t3 = sn * s * (c * c * c) * ok * (5.0 - (t * t) + 9.0 * eta + 4.0 _
        * (eta * eta)) / 24.0
    If (latRad < 0.0) Then nfn = 10000000.0 Else nfn = 0
    northing = nfn + t1 + (dlam * dlam) * t2 + (dlam * dlam * dlam _
        * dlam) * t3 + (dlam * dlam * dlam * dlam * dlam * dlam)
    t6 = sn * c * ok
    t7 = sn * (c * c * c) * (1.0 - (t * t) + eta) / 6.0
    easting = fe + dlam * t6 + (dlam * dlam * dlam) * t7
    If (northing >= 9999999.0) Then northing = 9999999.0
	res.x = easting
	res.Y = northing
	res.utmXZone = utmXZone
	Return res
End Sub


'Convert UTM to LatLon
'a - Equatorial radius
'f - Flattening
Public Sub UTMToLatLon( a As Double,  f As Double, U As UTM) As LatLon

    Dim recf As Double
    Dim b As Double
    Dim eSquared As Double
    Dim e2Squared As Double
    Dim tn As Double
    Dim ap As Double
    Dim bp As Double
    Dim cp As Double
    Dim dp As Double
    Dim ep As Double
    Dim olam As Double
    Dim nfn As Double
    Dim tmd As Double
    Dim sr As Double
    Dim sn As Double
    Dim ftphi As Double
    Dim s As Double
    Dim c As Double
    Dim t As Double
    Dim eta As Double
    Dim de As Double
    Dim dlam As Double
    Dim lat, lon As Double
    recf = 1.0 / f
    b = a * (recf - 1) / recf
    eSquared = CalculateESquared(a, b)
    e2Squared = CalculateE2Squared(a, b)
    tn = (a - b) / (a + b)
    ap = a * (1.0 - tn + 5.0 * ((tn * tn) - (tn * tn * tn)) / 4.0 + 81.0 * _
        ((tn * tn * tn * tn) - (tn * tn * tn * tn * tn)) / 64.0)
    bp = 3.0 * a * (tn - (tn * tn) + 7.0 * ((tn * tn * tn) _
        - (tn * tn * tn * tn)) / 8.0 + 55.0 * (tn * tn * tn * tn * tn) / 64.0) _
        / 2.0
    cp = 15.0 * a * ((tn * tn) - (tn * tn * tn) + 3.0 * ((tn * tn * tn * tn) _
        - (tn * tn * tn * tn * tn)) / 4.0) / 16.0
    dp = 35.0 * a * ((tn * tn * tn) - (tn * tn * tn * tn) + 11.0 _
        * (tn * tn * tn * tn * tn) / 16.0) / 48.0
    ep = 315.0 * a * ((tn * tn * tn * tn) - (tn * tn * tn * tn * tn)) / 512.0
    If Not (U.NorthHemisphere) Then
        nfn = 10000000.0
    Else
        nfn = 0
    End If
    tmd = (U.Y - nfn) / ok
    sr = sphsr(a, eSquared, 0.0)
    ftphi = tmd / sr
    Dim t10 As Double, t11 As Double, t14 As Double, t15 As Double
    Dim Index As Int
    For Index = 0 To 4
        t10 = sphtmd(ap, bp, cp, dp, ep, ftphi)
        sr = sphsr(a, eSquared, ftphi)
        ftphi = ftphi + (tmd - t10) / sr
    Next
    sr = sphsr(a, eSquared, ftphi)
    sn = sphsn(a, eSquared, ftphi)
    s = Sin(ftphi)
    c = Cos(ftphi)
    t = s / c
    eta = e2Squared * (c * c)
    de = U.X - fe
    t10 = t / (2.0 * sr * sn * (ok * ok))
    t11 = t * (5.0 + 3.0 * (t * t) + eta - 4.0 * (eta * eta) - 9.0 * (t * t) _
        * eta) / (24.0 * sr * (sn * sn * sn) * (ok * ok * ok * ok))
    lat = ftphi - (de * de) * t10 + (de * de * de * de) * t11
    t14 = 1.0 / (sn * c * ok)
    t15 = (1.0 + 2.0 * (t * t) + eta) / (6 * (sn * sn * sn) * c _
        * (ok * ok * ok))
    dlam = de * t14 - (de * de * de) * t15
    olam = (U.UtmXZone * 6 - 183.0) * deg2rad
    lon = olam + dlam
    lon = lon * rad2deg
    lat = lat * rad2deg
	Dim ll As LatLon
	ll.Initialize
	ll.lat = lat
	ll.lon = lon
    Return ll
End Sub
	
'Public Sub ChangeDatum( LL As LatLon, _
' 	From_a As Double,  From_f As Double,  To_a As Double,  To_f As Double, _
' 	dx As Double,  dy As Double,  dz As Double) As LatLon
'	Dim lat = ll.Lat, lon = ll.Lon As Double
'    lat = lat / 180 * cPI
'    lon = lon / 180 * cPI
'    Dim slat As Double = Sin(lat)
'    Dim clat As Double = Cos(lat)
'    Dim slon As Double = Sin(lon)
'    Dim clon As Double = Cos(lon)
'    Dim ssqlat As Double = slat * slat
'    Dim adb As Double = 1.0 / (1.0 - From_f) '  // "a divided by b"
'    Dim dlat, dlon As Double
'    Dim from_esq As Double = 2 * From_f - Power(From_f, 2)
'    Dim rn As Double = From_a / Sqrt(1.0 - from_esq * ssqlat)
'    Dim rm As Double = From_a * (1.0 - from_esq) / Power((1.0 - from_esq * ssqlat), 1.5)
'    Dim res As LatLon
'    Dim da As Double = To_a - From_a
'    Dim df As Double = To_f - From_f
'    dlat = (((((-dx * slat * clon - dy * slat * slon) + dz * clat) _
'                + (da * ((rn * from_esq * slat * clat) / From_a))) _
'            + (df * (rm * adb + rn / adb) * slat * clat))) _
'        / (rm + 0)
'
'    dlon = (-dx * slon + dy * clon) / ((rn + 0) * clat)
'
'  	res.Initialize
'    res.Lat = lat + dlat
'    res.Lon = lon + dlon
'    res.Lat = res.Lat / cPI * 180
'    res.lon = res.Lon / cPI * 180
'    Return res
'End Sub


