﻿Type=StaticCode
Version=0.1
ModulesStructureVersion=1
B4i=true
@EndOfDesignText@
'Code module

Sub Process_Globals
	'These global variables will be declared once when the application starts.
	'Public variables can be accessed from all modules.
	Private rtPage As Page
	Private LocManager As LocationManager
	Private lblAccuracy As Label
	Private lblAltitude As Label
	Private lblBearing As Label
	Private lblEnable As Label
	Private lblHeading As Label
	Private lblLL As Label
	Private lblSpeed As Label
	Private lblTime As Label
	Private lblUTM As Label
	Private btnCopyToClipboard As Button
	Private cp As Clipboard
	Private pnlUTM As Panel
	Private message As MessageComposer
	Private mail As MailComposer
	Private smsButton, mailButton As BarButton
End Sub

Public Sub Show
	If rtPage.IsInitialized = False Then
		rtPage.Initialize("rtPage")
		rtPage.RootPanel.LoadLayout("2")
		LocManager.Initialize("LocManager")
		message.Initialize("message")
		mail.Initialize("mail")
		smsButton = rtPage.ToolbarButtons.Get(0)
		mailButton = rtPage.ToolbarButtons.Get(2) 'there is a spacer between each two buttons
	End If
	Main.NavControl.ShowPage(rtPage)
	
End Sub

Public Sub Application_Background
	If rtPage.IsInitialized AND rtPage.Visible Then
		RtPage_Disappear
	End If
End Sub

Public Sub Application_Foreground
	If rtPage.IsInitialized AND rtPage.Visible Then
		RtPage_Appear
	End If
End Sub

Public Sub RtPage_Appear
	If LocManager.IsAuthorized OR LocManager.AuthorizationStatus = LocManager.AUTHORIZATION_NOT_DETERMINED Then
		LocManager.Start(0)
	End If
	LocManager.StartHeading
	
	
End Sub

Public Sub RtPage_Disappear
	LocManager.Stop
	LocManager.StopHeading
End Sub

Private Sub LocManager_AuthorizationStatusChanged (Status As Int)
	lblEnable.Visible = (LocManager.AuthorizationStatus = LocManager.AUTHORIZATION_DENIED _
		OR LocManager.AuthorizationStatus = LocManager.AUTHORIZATION_RESTRICTED)
	
End Sub

Private Sub LocManager_HeadingChanged (MagneticHeading As Double, TrueHeading As Double)
	If TrueHeading >= 0 Then
		lblHeading.Text = NumberFormat(TrueHeading, 1, 0) & Chr(176) & " (true north)"
	Else
		lblHeading.Text = NumberFormat(MagneticHeading, 1, 0) & Chr(176) & " (magnetic)"
	End If
	
End Sub

Private Sub LocManager_LocationError
	Log("Error: " & LastException.Description)
End Sub

Private Sub LocManager_LocationChanged (Location1 As Location)
	If Location1.VerticalAccuracy >= 0 Then
		lblAltitude.Text =	NumberFormat(Location1.Altitude, 1, 1) & "m"
	Else
		lblAltitude.Text = "N/A"
	End If
	lblBearing.Text = ValueOrNA (Location1.Bearing, 1, Chr(176))
	If Location1.Accuracy >= 0 Then
		lblLL.Text = Main.LatLonToString(Location1.Latitude) & " / " & _
			 Main.LatLonToString(Location1.Longitude)
		Dim ll As LatLon
		ll.Initialize
		ll.Lat = Location1.Latitude
		ll.Lon = Location1.Longitude
		Dim U As UTM = Main.geo.LatLonToUtm(Main.selectedDatum.A,Main.selectedDatum.f, ll)
		lblUTM.Text = U.UtmXZone & "  " & NumberFormat2(U.X, 0, 0, 0, False) & " " _
			& NumberFormat2(U.Y, 0, 0, 0, False)
		If btnCopyToClipboard.Enabled = False Then
			btnCopyToClipboard.Enabled = True
			smsButton.Enabled = message.CanSendText
			mailButton.Enabled = mail.CanSendMail
		End If
	Else
		lblLL.Text = "N/A"
		lblUTM.Text = "N/A"
		btnCopyToClipboard.Enabled = False
		smsButton.Enabled = False
		mailButton.Enabled = False
	End If
	lblSpeed.Text = ValueOrNA(Location1.Speed, 1, "m/s")
	lblTime.Text = DateTime.Time(Location1.Time)
	lblAccuracy.Text = ValueOrNA(Location1.Accuracy, 2, "m")
End Sub

Private Sub ValueOrNA(value As Double, NumberOfFractions As Int, unit As String) As String
	If value < 0 Then
		Return "N/A"
	Else
		Return NumberFormat(value, 1, NumberOfFractions) & unit
	End If
End Sub

Sub btnCopyToClipboard_Click
	cp.StringItem = lblUTM.Text
	pnlUTM.Color = Colors.Green
	pnlUTM.SetColorAnimated(1000, Colors.Transparent)
End Sub

Sub rtPage_BarButtonClick (Tag As String)
	If Tag = "SMS" Then
		message.Initialize("message")
		message.Body = "My location: " & lblUTM.Text
		message.Show(rtPage)
	Else If Tag = "Email" Then
		mail.Initialize("mail") 'do not reuse the same mail object. It doesn't work.
		mail.SetSubject("My location")
		mail.SetBody(lblUTM.Text, False)
		mail.Show(rtPage)
	End If
End Sub

Sub Mail_Complete (Result As Int)
	
End Sub
Sub Message_Complete (Result As Int)
	
End Sub